CON
           
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip I2C Logger Demonstration                                  AiChip_I2cLogger_Demo_005      *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     Object Name    : AiChip I2C LoggerDemonstration                                                 *
' *     Version        : 005                                                                            *
' *     Updated        : 2008-04-13                                                                     *
' *                                                                                                     *
' *     Author         : AiChip Industries                                                              *
' *                                                                                                     *
' *     Copyright      : 2008, AiChip Industries                                                        *
' *                                                                                                     *
' *     Target Chip    : Parallax Propeller Chip P8X32A ( "Propeller Mk I" )                            *
' *     Compiler Used  : Parallax Propeller Tool 1.1                                                    *
' *                                                                                                     *
' *     License        : MIT License. Please see end of this file for Terms of Use.                     *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip is a trademark of AiChip Industries.                                                     *
' *                                                                                                     *
' *     If distributing a modified version of this software AiChip Industries requests the "AiChip"     *
' *     prefix be removed from the filename and internal text descriptions to avoid any confusion that  *
' *     may be caused by an impression that the modified software originates from AiChip Industries.    *
' *                                                                                                     *
' *******************************************************************************************************

PRI Version

  result := String( "AiChip_I2cLogger_Demo_005" )
  
CON

  _CLKMODE      = XTAL1 + PLL16x
  _XINFREQ      = 5_000_000

  TV_PIN        = 12
  
  SDA_PIN       = 29
  SCL_PIN       = 28

  ID            = $10_7E2
  
OBJ

  tv            : "TV_Text"
  
  i2c_Logger    : "AiChip_I2cSlave_Logger_005"
  i2c_Slave     : "AiChip_I2cSlave_Slave_005"
  i2c_Master    : "AiChip_I2cSlave_Master_005"

CON

  LOG_SIZE = 1000

VAR

  long logData[ LOG_SIZE+1 ]
    
PUB Demo | logPtr, bitCount, allCount

  tv.Start( TV_PIN )

  ' Start the I2C Slave running. It uses the same I2C lines as the master does so no wiring needed.

  i2c_Logger.Start( SCL_PIN, SDA_PIN, ID, @logData, LOG_SIZE )

  WaitCnt( CLKFREQ / 1000 * 100 + CNT )

  i2c_Slave.Start( SCL_PIN, SDA_PIN, ID, 0, 0 )

  ' Allow the I2C Slave time to start so we can be sure that starting the I2C Master doesn't cause it any
  ' problems when kicking the I2C bus into life.
  
  WaitCnt( CLKFREQ / 1000 * 100 + CNT )

  ' Initialise the I2C Master code so it knows what it's about to communicate with.
  
  i2c_Master.OpenRaw( SCL_PIN, SDA_PIN, ID, 0, 0 )

  i2c_Master.WriteByte($1234,$56)
  i2c_Master.ReadByte($1234)

  ' Logged data is ...          1111 1111 1111 1111 1111 1111 1111 1111     Not used
  '                             0000 0000 0000 0000 0000 0000 0nnn nnnn     Character "S", "0", "1", "E"
  '                             0000 0000 0000 0000 0000 0000 0000 00cd     Sclk / Sda data bits
  '                             0100 0000 0000 0000 0000 0000 00cd cdcd     Sclk / Sda Triplet
  '                             1000 0000 0000 0000 0000 0000 nnnn nnnn     Byte data
  '                             1100 0000 0000 0000 nnnn nnnn nnnn nnnn     Word data

  logPtr := 0
  bitcount := -1
  repeat                                
    repeat while ( result := logData[ logPtr ] ) == -1
 
    if ( result & $FFFF_FF00 ) == $0000_0000
      if ( result & $FFFF_FFFC ) == $0000_0000
        ' tv.Out( " " )
        ' tv.Bin( result, 2 ) 
      else
        case result
        
          "S"      : tv.Out( $0D )
                     tv.Out( result )
                     bitCount := 0
                     allCount := 0
                     
          "P", "E" : tv.Out( " " )
                     tv.Out( result )
                     bitCount := -1
                     allCount := -1
                     
          "0", "1" : if bitCount => 0
                       bitCount++
                       allCount++
                       if bitCount == 1
                         tv.Out( " " )
                       if allCount == 8
                         if result == "0"
                           tv.Out( "W" )
                         else
                           tv.Out( "R" )
                       else
                         if bitCount == 9
                           tv.Out( " " )
                           if result == "0"
                             tv.Out( "A" )
                           else
                             tv.Out( "N" )
                           bitCount := 0
                         else
                           tv.Out( result )
                     else
                       tv.Out( result )
                       
          other    : tv.Out( result )
                     allCount := -1
                     
    if ( result & $C000_0000 ) == $4000_0000
      tv.Out( "<" )
      tv.Bin( result >> 4, 2 )
      tv.Out( "_" )
      tv.Bin( result >> 2, 2 )
      tv.Out( "_" )
      tv.Bin( result >> 0, 2 )
      tv.Out( ">" )
      
    if ( result & $C000_0000 ) == $8000_0000
      tv.Out( " " )
      tv.Hex( result, 2 ) 

    if ( result & $C000_0000 ) == $C000_0000
      tv.Out( $0D )
      tv.Hex( result, 4 ) 

    logPtr++
    

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     Terms of Use : MIT License                                                                      *
' *                                                                                                     *
' *******************************************************************************************************
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
' associated documentation files (the "Software"), to deal in the Software without restriction, including
' without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to
' the following conditions:                                                                   
'
' The above copyright notice and this permission notice shall be included in all copies or substantial
' portions of the Software.                                                                                             
'                                                                                                                  
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
' NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
' WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip I2C Logger Demonstration                                                          *
' *                                                                                                     *
' *******************************************************************************************************
        