CON
           
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip I2C Slave Demonstration                                    AiChip_I2cSlave_Demo_005      *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     Object Name    : AiChip I2C Slave Demonstration                                                 *
' *     Version        : 005                                                                            *
' *     Updated        : 2008-04-13                                                                     *
' *                                                                                                     *
' *     Author         : AiChip Industries                                                              *
' *                                                                                                     *
' *     Copyright      : 2008, AiChip Industries                                                        *
' *                                                                                                     *
' *     Target Chip    : Parallax Propeller Chip P8X32A ( "Propeller Mk I" )                            *
' *     Compiler Used  : Parallax Propeller Tool 1.1                                                    *
' *                                                                                                     *
' *     License        : MIT License. Please see end of this file for Terms of Use.                     *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip is a trademark of AiChip Industries.                                                     *
' *                                                                                                     *
' *     If distributing a modified version of this software AiChip Industries requests the "AiChip"     *
' *     prefix be removed from the filename and internal text descriptions to avoid any confusion that  *
' *     may be caused by an impression that the modified software originates from AiChip Industries.    *
' *                                                                                                     *
' *******************************************************************************************************

' *******************************************************************************************************
' *                                                                                                     *
' *     HISTORY                                                                                         *
' *                                                                                                     *
' *     005     2008-04-13      Multiple I2C Slaves and Real Eeprom tested - 10-bit addressing added    *
' *                                                                                                     *
' *     004     2008-04-12      Uses latest PASM version of I2C Slave - Working, faster sampling        *
' *                                                                                                     *
' *     003     2008-04-12      Uses latest PASM version of I2C Slave - Working                         *
' *                                                                                                     *
' *     002     2008-04-12      PASM version of I2C Slave included - Not working yet                    *
' *                             Improved I2C Master control line handling                               *
' *                                                                                                     *
' *     001     2008-04-11      First Public Release                                                    *
' *                                                                                                     *
' *******************************************************************************************************

PRI Version

  result := String( "AiChip_I2cSlave_Demo_005" )
  
CON

  _CLKMODE      = XTAL1 + PLL16x
  _XINFREQ      = 5_000_000

  TV_PIN        = 12
  
  SDA_PIN       = 29
  SCL_PIN       = 28

  ID_1          =     $E0                               ' Set to -1, $10..$EE, $10_000..$10_7FE
  ID_2          =     $E2
  ID_3          = $10_0E2
  
OBJ

  ' These are used by this program
  
  tv            : "TV_Text"
  
  i2c_Slave1    : "AiChip_I2cSlave_Slave_005"
  i2c_Slave2    : "AiChip_I2cSlave_Slave_005"
  i2c_Slave3    : "AiChip_I2cSlave_Slave_005"
  
  i2c_Master    : "AiChip_I2cSlave_Master_005"

OBJ

  ' These are included simply to get the .spin files into the Archive .ZIP package
  
  add_LogDemo   : "AiChip_I2cLogger_Demo_005"
  add_Logger    : "AiChip_I2cSlave_Logger_005"
  add_32Kx8     : "AiChip_I2cSlave_32Kx8Ram_005"
  add_16Kx8     : "AiChip_I2cSlave_16Kx8Ram_005"
  
PUB Demo | testNum, pc, n, repeatCount, testRealEeprom

  tv.Start( TV_PIN )

  ' Start the I2C Slaves running. It uses the same I2C lines as the master does so no wiring needed.

  if ID_1 => 0
    i2c_Slave1.Start( SCL_PIN, SDA_PIN, ID_1, 0, 0 )
  if ID_2 => 0
    i2c_Slave2.Start( SCL_PIN, SDA_PIN, ID_2, 0, 0 )
  if ID_3 => 0
    i2c_Slave3.Start( SCL_PIN, SDA_PIN, ID_3, 0, 0 )
  
  ' Allow the I2C Slaves time to start so we can be sure that starting the I2C Master doesn't cause it any
  ' problems when kicking the I2C bus into life.
  
  WaitCnt( CLKFREQ / 1000 * 100 + CNT )

  ' Test genuine Eeprom works
   
  i2c_Master.OpenRaw( 28, 29, $A0, 0, 5 )
   
  Header
  if not ( testRealEeprom := i2c_Master.ReadLong( @magic ) == $D0D0_F00D )
    tv.Str( String( $0D,$0D,"FAILED REAL EEPROM TEST !",$0D,"Possibly running in RAM ?") )
    tv.Str( String( $0D,$0D,"Testing will start in 10 seconds ..." ) )
    waitcnt( CLKFREQ * 10 +CNT )
    tv.Out( $00 )
  else
    i2c_Master.WriteLong( @magic, ! $D0D0_F00D )
    if ( result := i2c_Master.ReadLong( @magic ) ) <> ! $D0D0_F00D
      tv.Str( String( $0D,$0D,"FAILED EEPROM INVERT",$0D,$0D,"Read " ) )
      tv.Hex( result, 8 )
      tv.Str( String( $0D,"Want ") )
      tv.Hex( ! $D0D0_F00D, 8 )
      repeat
    i2c_Master.WriteLong( @magic, $D0D0_F00D )
    if ( result := i2c_Master.ReadLong( @magic ) ) <> $D0D0_F00D
      tv.Str( String( $0D,$0D,"FAILED EEPROM RESTORE",$0D,$0D,"Read " ) )
      tv.Hex( result, 8 )
      tv.Str( String( $0D,"Want ") )
      tv.Hex( $D0D0_F00D, 8 )
      repeat
    
  ' Test every possible pattern in the 1024 byte I2C Slave Ram

  repeatCount := 1024
      
  repeat testNum from $00 to $FF

    Header
    tv.Str( String( $0D,$0D,"Test : " ) )
    tv.Hex( testNum, 2 )
    if testNum
      tv.Str( String( " - Okay so far" ) )
      
    pc := 0
    n  := testNum
    repeat repeatCount
      if ID_1 => 0
        i2c_Master.OpenRaw( SCL_PIN, SDA_PIN, ID_1, 0, 0 )
        i2c_Master.WriteByte( pc, n+0 )
      if ID_2 => 0  
        i2c_Master.OpenRaw( SCL_PIN, SDA_PIN, ID_2, 0, 0 )
        i2c_Master.WriteByte( pc, n+1 )
      if ID_3 => 0
        i2c_Master.OpenRaw( SCL_PIN, SDA_PIN, ID_3, 0, 0 )
        i2c_Master.WriteByte( pc, n+2 )
      pc++
      if ( n++ ) == 0
        n++
      
    repeat 2
      pc := 0
      n  := testNum
      repeat repeatCount
        if testRealEeprom
          i2c_Master.OpenRaw( 28, 29, $A0, 0, 5 )
          if i2c_Master.ReadLong( @magic )<> $D0D0_F00D
            tv.Str( String( $0D,$0D,"FAILED REAL EEPROM" ) )
            repeat
        if ID_1 => 0
          i2c_Master.OpenRaw( SCL_PIN, SDA_PIN, ID_1, 0, 0 )
          Check( 1, pc, n+0 )
        if ID_2 => 0
          i2c_Master.OpenRaw( SCL_PIN, SDA_PIN, ID_2, 0, 0 )
          Check( 2, pc, n+1 )
        if ID_2 => 0
          i2c_Master.OpenRaw( SCL_PIN, SDA_PIN, ID_3, 0, 0 )
          Check( 3, pc, n+2 )
        pc++
        if ( n++ ) == 0
          n++

  tv.Str( String( $0D,$0D,"PASSED ALL TESTS") )

PRI Header

  tv.Out( $01 )
  tv.Str( Version )
  tv.Out( $0D )
  if ID_1 => 0
    tv.Str( i2c_Slave1.Version )
    ShowId( 1, ID_1 )
  if ID_2 => 0
    tv.Str( i2c_Slave2.Version )
    ShowId( 2, ID_2 )
  if ID_3 => 0
    tv.Str( i2c_Slave3.Version )
    ShowId( 3, ID_3 )
  tv.Str( i2c_Master.Version )

PRI ShowId( n, id )

  tv.Str( String( " : " ) )
  tv.Dec( n )
  tv.Str( String( " = " ) )
  if id & $10_000
    tv.Str( String( "10_" ) )
    tv.hex( id, 3 )
  else
    tv.Hex( id, 2 )
  tv.Out( $0D )
  
PRI Check( which, pc, n )

  n &= $FF
  
  if ( result := i2c_Master.ReadByte( pc ) ) <> n
     tv.Str( String( $0D,$0D,"FAILED " ) )
     tv.Dec( which )
     tv.Str( String( " : At " ) )
     tv.Hex( pc, 4 )
     tv.Str( String( " Wrote ") )
     tv.Hex( n , 2 )
     tv.Str( String( " Read ") )
     tv.Hex( result, 2 )
     repeat
          
DAT

magic   long  $D0D0_F00D
    
CON

' *******************************************************************************************************
' *                                                                                                     *
' *     Terms of Use : MIT License                                                                      *
' *                                                                                                     *
' *******************************************************************************************************
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
' associated documentation files (the "Software"), to deal in the Software without restriction, including
' without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to
' the following conditions:                                                                   
'
' The above copyright notice and this permission notice shall be included in all copies or substantial
' portions of the Software.                                                                                             
'                                                                                                                  
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
' NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
' WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip I2C Slave Demonstration                                                           *
' *                                                                                                     *
' *******************************************************************************************************
        